<?php
// $Id: smartytemplate.php,v 1.5.2.2 2005/12/31 07:50:17 tclineks Exp $

/**
 * @file
 *   Class Definition for SmartyTemplate - interface between smarty engine and smarty-based themes
 *
 * Originally derived from the phptemplate version by Brian E. Lozier (brian@massassi.net)
 *
 * @author
 *   tclineks - Travis Cline (travis dot cline at gmail dot com)
 * @author
 *   lobo - Donald A. Lobo (lobo at yahoo dot com)
 * @license
 *   http://www.gnu.org/copyleft/gpl.html Licensed under GPL
 */

define('ENGINE_PATH', 'themes/engines/smarty');
require_once realpath(ENGINE_PATH.'/inc/common.php'); 

// Smarty Drupal engine, by default, expects a smarty installation in ./libs
if (!class_exists('Smarty')) {  // prevent redeclaration
  $smarty_path = realpath(ENGINE_PATH.'/libs/Smarty.class.php');
  if ($smarty_path != false)
    include_once $smarty_path; 
  else
    echo 'Smarty could not be found for inclusion. Please refer to the documentation at http://www.drupal.org/project/smarty';
}

class SmartyTemplate extends Smarty {
  var $path; // Path to the templates
  var $vars; // Shorcut to _tpl_vars at runtime

  /**
   * Constructor
   *
   * @param string $path the path to the templates
   *
   * @return void
   */
  function SmartyTemplate($path = null) {
    $this->path = $path;

    $this->Smarty();

    $this->vars = & $this->_tpl_vars;
    $this->template_dir = $path;
    $this->compile_dir  = $this->_get_compile_dir($path);
    // $this->compile_dir  = '/absolute/path/here';
    $this->config_dir   = $path.'/configs';
    $this->plugins_dir  = array(realpath('themes/engines/smarty/plugins'), $path.'/plugins', 'plugins');
  }

  /**
   * Set the path to the template files.
   *
   * @param string $path path to template files
   *
   * @return void
   */
  function set_path($path) {
    $this->path = $path;
  }

  /**
   * Set a template variable.
   *
   * @param string $name name of the variable to set
   * @param mixed $value the value of the variable
   *
   * @return void
   */
  function set($name, $value) {
    $this->assign($name, $value);
  }

  /**
   * Set a bunch of variables at once using an associative array.
   *
   * @param array $vars array of vars to set
   * @param bool $clear whether to completely overwrite the existing vars
   *
   * @return void
   */
  function set_vars($vars, $clear = false) {
    if ($clear) {
      $this->clear_all_assign();
    }
    foreach ($vars as $name => $value) {
      $this->assign($name, $value);
    }
  }

  /**
   * Open, parse, and return the template file.
   *
   * @param string string the template file name
   *
   * @return string
   */
  function fetch($file, $cache_id = null, $compile_id = null) {
    $this->_register_wrappers();
    return parent::fetch($file, $cache_id, $compile_id);              // Return the contents
  }

  /**
   * Performs runtime registration of functions and modifiers to enable ease of use
   * with in templates.
   *
   * Theme-specific smartytemplate.php files can publish a smarty_register_functions
   * and a smarty_register_modifiers 
   * function which returns an array of function names to register.
   * 
   * If any elements are associative the key will be used as the function
   * name published to Smarty while the value refers to the existing function.
   */
  function _register_wrappers() {
    include_once 'inc/default_wrappers.php';
    $functions = _get_default_functions();
    $modifiers = _get_default_modifiers();
    
    if (function_exists('smarty_register_functions'))
      $functions = array_merge($functions, smarty_register_functions());

    if (function_exists('smarty_register_modifiers'))
      $modifiers = array_merge($modifiers, smarty_register_modifiers());

    foreach ($functions as $key => $value) {
      if (is_numeric($key))
        $key = $value;  // handle single parameter
      $this->register_function($key, $value);
    }

    foreach ($modifiers as $key => $value) {
      if (is_numeric($key))
        $key = $value;  // handle single parameter
      $this->register_modifier($key, $value);
    }
  }

  /**
   * Returns compile dir, attempts to create it if it doesn't exist.
   * If unable to create it will stop execution and direct user to permissions setup
   */
  function _get_compile_dir($path) {
    $dir = ENGINE_PATH.'/templates_c/'.substr(strrchr($path, '/'), 1);
    if (!file_exists($dir)) {
      if (!@mkdir($dir)) {
        _test_permissions(true);
        exit;
      }
    }    
    return $dir;
  }

}

?>
